﻿using System;
using System.Collections.Generic;
using System.IO;
using UnityEditor;
using UnityEngine;

public class CreateAssetBundles : MonoBehaviour
{
    static private string Dream_UnityVersion { get; set; }

    static public bool CheckUnityVersion(string exporterVersion)
    {
        string path = Path.Combine(Application.dataPath, "Threespin", "Dream_UnityVersion.json");

        if (!File.Exists(path))
        {
            Debug.LogError(path + " not found.");
            Dream_UnityVersion = "*file missing*";
            return false;
        }

        string dreamVersion = string.Empty;

        using (StreamReader r = new StreamReader(path))
        {
            string json = r.ReadToEnd();
            dreamVersion = JsonUtility.FromJson<JsonObject>(json).Dream_UnityVersion;
            Dream_UnityVersion = dreamVersion;
        }
        return CompareVersion(exporterVersion, dreamVersion);
    }

    public static bool CompareVersion(string versionToCheck, string versionWeWantToCompareTo) 
    { 
        string[] exporterVersionPart = versionToCheck.Split('.');
        string[] dreamVersionPart = versionWeWantToCompareTo.Split('.');     

        int exporterMajorVersion = int.Parse(exporterVersionPart[0]);
        int dreamMajorVersion = int.Parse(dreamVersionPart[0]);    
        
        if (exporterMajorVersion != dreamMajorVersion)
        {       
            return false;
        }

        int exporterMinorVersion = int.Parse(exporterVersionPart[1]);
        int dreamMinorVersion = int.Parse(dreamVersionPart[1]);
       
        if (exporterMinorVersion != dreamMinorVersion)
        {      
            return false;
        }
       
        return true;
    }


    static public bool ShowWarningDialogue(string exporterVersion)
    {
        return EditorUtility.DisplayDialog("Wrong Unity version (" + exporterVersion + ")",
               "You are using the wrong Unity version. \n" +
               "\n" +
               "Please use Unity update " + Dream_UnityVersion +" to prevent compatibility problems with 3spin DREAM."
                , "Ok", "Cancel");
    }   

    static Dictionary<BuildTarget, string> PlatformBundleNames = new Dictionary<BuildTarget, string>
    {
        [BuildTarget.Android] = ".androidbundle",
        [BuildTarget.StandaloneWindows] = ".winbundle",
        [BuildTarget.WSAPlayer] = ".uwpbundle"
    };

    [MenuItem("Assets/Build AssetBundles")]
    static void BuildAllAssetBundles()
    {
        string exporterVersion = Application.unityVersion;

        if (!CheckUnityVersion(exporterVersion))
        {
            if (!ShowWarningDialogue(exporterVersion))
            {
                Debug.Log("Build process canceled.");
                return;
            }
        }

        string assetBundleDirectory = "Assets/Threespin/DREAM/AssetBundles/";

        foreach (KeyValuePair<BuildTarget, string> platform in PlatformBundleNames)
        {
            BuildBundleForPlatform(assetBundleDirectory + platform.Key.ToString(), platform.Key, platform.Value);
        }
    }

    static void BuildBundleForPlatform(string directory, BuildTarget buildTarget, string fileExtension)
    {
        if (!Directory.Exists(directory))
        {
            Directory.CreateDirectory(directory);
        }

        if (buildTarget == BuildTarget.Android)
        {
            PlayerSettings.stereoRenderingPath = StereoRenderingPath.SinglePass;
        }
        else
        {
            PlayerSettings.stereoRenderingPath = StereoRenderingPath.Instancing;
        }

        BuildPipeline.BuildAssetBundles(directory, BuildAssetBundleOptions.None, buildTarget);

        foreach (string file in Directory.GetFiles(directory))
        {
            if (Path.GetExtension(file).Equals("bundle") || Path.GetExtension(file).Equals(""))
            {
                string fileDestinationPath = file + fileExtension;
                Debug.Log("Writing assetBundle: " + fileDestinationPath);
                if (File.Exists(fileDestinationPath))
                {
                    Debug.LogWarning("File at path " + fileDestinationPath + " already exists but Bundle changed since then. Overriding...");
                    File.Delete(fileDestinationPath);
                }
                File.Move(file, file + fileExtension);
            }
        }

        Debug.Log("Creation of AssetBundles completed.");
    }

    [Serializable]
    class JsonObject
    {
        public string Dream_UnityVersion = string.Empty;
    }
}   

